/* ------------------------------ hellowin.c ----------------------------- */
/*  description: This program displays the text "Hello, Windows!" centered
 *               on the client area of a screen.  This should be viewed as
 *               the equivalent of the "Hello World" program that is used
 *               traditionally as the first program in a learning process.
 *
 *       source: Programming Windows by Charles Petzold, 1990, Chapter 1
 *
 *     modified: 02/27/92 by Flavelle Ballem.
 *
 *               The program is an example of an implementation of a task
 *               switching mechanism.  This approach allows eliminates the
 *               lengthy switch statements that are found in most windows
 *               functions and encourages a more modular approach to the
 *               development of the program interface.
 *
 *               The mechanism is non-vendor specific, and can be applied
 *               to programs in C and C++.
 */

#include <windows.h>

// definition of the function type that is required for the task list

#define TASKPROC  LONG FAR PASCAL

/* tTaskEntry
 *
 *         This structure type is used to define a single entry in a windows
 *         task list.  The members of this structure are:
 *
 *              msg     The value of the message that is to be processed by
 *                      the task switch.
 *              fnc     The address of the function that is to process the
 *                      message.  The function declaration must be:
 *
 *                          TASKPROC FuncName (HWND, WORD, WORD, LONG);
 *
 *                      where FuncName is the name of the function that will
 *                      be used to process the message.
 */

typedef struct
        {
        WORD msg;
        LONG (FAR PASCAL *fnc)(HWND, WORD, WORD, LONG);
        } tTaskEntry;


/* function declarations
 *
 *         WndProc     The main message processor.  In a traditional windows
 *                     program, a switch() statement is used, and would trap
 *                     and process the messages.  In this program, the task
 *                     switch is implemented.
 *
 *         tfWMPaint   This function is used to process WM_PAINT messages.
 *
 *         tfWMDestroy This function is used to process WM_DESTROY messages.
 */

long FAR PASCAL WndProc(HWND,WORD,WORD,LONG);
TASKPROC tfWMPaint (HWND, WORD, WORD, LONG);
TASKPROC tfWMDestroy (HWND, WORD, WORD, LONG);

/* WinMain()
 *
 *         This is the required windows procedure.  It is used to register
 *         the window class (for the first instance of the application) and
 *         to create and display the window.  In addition, it sets up the
 *         message loop.
 *
 *         The lpszCmdLine parameter is not used.  The "argsused" directive
 *         is applied to eliminate the warning message.
 */

#pragma argsused

int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance,
                    LPSTR lpszCmdParam, int nCmdShow)
    {
    static char szAppName[] = "HelloWin";
    HWND        hwnd;
    MSG         msg;
    WNDCLASS    wndclass;


    if  (!hPrevInstance)
        {
        wndclass.style          = CS_HREDRAW | CS_VREDRAW;
        wndclass.lpfnWndProc    = WndProc;
        wndclass.cbClsExtra     = 0;
        wndclass.cbWndExtra     = 0;
        wndclass.hInstance      = hInstance;
        wndclass.hIcon          = LoadIcon(NULL, IDI_APPLICATION);
        wndclass.hCursor        = LoadCursor(NULL, IDC_ARROW);
        wndclass.hbrBackground  = COLOR_WINDOW + 1;
        wndclass.lpszMenuName   = NULL;
        wndclass.lpszClassName  = szAppName;
        RegisterClass (&wndclass);
        }

    hwnd = CreateWindow ( szAppName,
                          "The Hello Program",
                          WS_OVERLAPPEDWINDOW,
                          CW_USEDEFAULT,
                          CW_USEDEFAULT,
                          CW_USEDEFAULT,
                          CW_USEDEFAULT,
                          NULL,
                          NULL,
                          hInstance,
                          NULL );


    ShowWindow(hwnd,nCmdShow);
    UpdateWindow(hwnd);

    while (GetMessage (&msg,NULL,0,0))
          {
          TranslateMessage(&msg);
          DispatchMessage(&msg);
          }

    return msg.wParam;
    }   /* end of WinMain Function */

/* WndProc()
 *
 *         In a traditional windows program, this function would contain a
 *         switch statement with a case for all of the messages that we would
 *         process.  In this program, the task switch is executed.
 *
 *         This function must be exported.
 *
 *         The function works by defining the array of tTaskEntry structures.
 *         Note that the last element in the structure is for WM_NULL and
 *         the default window procedure (DefWindowProc) is specified.  This
 *         is required.  A while statement is executed to select the correct
 *         function that will be used to process the message.  The same
 *         function can be used to process more than one message.
 */

long FAR PASCAL WndProc (HWND hwnd, WORD message, WORD wParam, LONG lParam)
     {
     tTaskEntry TaskList[] =
                {
                {WM_PAINT,  tfWMPaint},
                {WM_DESTROY,tfWMDestroy},
                {WM_NULL,   DefWindowProc}
                };

     int i = 0;     // index used to reference the task list

     while ((TaskList[i].msg != NULL) && (TaskList[i].msg != message))
           i++;
     return ((*TaskList[i].fnc)(hwnd, message, wParam, lParam));
     }


/* tfWMPaint()
 *
 *         This function is called in response to a WM_PAINT message.  It
 *         is used to display a single line of text on the display which is
 *         centered, both vertically and horizontally.
 *
 *         The message, wParam, and lParam parameters are not used.  An
 *         "argsused" directive is applied to eliminate the warning message.
 */

#pragma argsused

TASKPROC tfWMPaint (HWND hwnd, WORD message, WORD wParam, LONG lParam)
         {
         HDC          hdc;
         PAINTSTRUCT  ps;
         RECT         rect;

         hdc = BeginPaint(hwnd,&ps);
         GetClientRect(hwnd,&rect);
         DrawText( hdc,"Hello Windows!",-1,&rect, DT_SINGLELINE | DT_CENTER |
                   DT_VCENTER);
         EndPaint(hwnd,&ps);
         return 0L;
         }

/* tfWMDestroy()
 *
 *         This function is called in response to a WM_DESTROY message.  It
 *         notifies windows that it is OK to terminate this application.
 *
 *         None of the parameters are used.  The "argsused" directive is
 *         applied to eliminate the warning messages.
 */

#pragma argsused

TASKPROC tfWMDestroy (HWND hwnd, WORD message, WORD wParam, LONG lParam)
         {
         PostQuitMessage(0);
         return 0L;
         }
